# Generated by Django 5.2 on 2024-09-16 17:17

import bionty.ids
import django.db.models.deletion
import lamindb.base.users
import lamindb.models
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0039_alter_cellline_source_alter_cellmarker_source_and_more"),
        ("lamindb", "0066_alter_artifact__feature_values_and_more"),
        ("wetlab", "0021_alter_compoundtreatment_duration_unit_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="experiment",
            name="experiment_type",
        ),
        migrations.RemoveField(
            model_name="biosample",
            name="collections",
        ),
        migrations.RemoveField(
            model_name="compoundtreatment",
            name="duration",
        ),
        migrations.RemoveField(
            model_name="compoundtreatment",
            name="duration_unit",
        ),
        migrations.RemoveField(
            model_name="compoundtreatment",
            name="ontology_id",
        ),
        migrations.RemoveField(
            model_name="compoundtreatment",
            name="pubchem_id",
        ),
        migrations.RemoveField(
            model_name="environmentaltreatment",
            name="duration",
        ),
        migrations.RemoveField(
            model_name="environmentaltreatment",
            name="duration_unit",
        ),
        migrations.RemoveField(
            model_name="experiment",
            name="collections",
        ),
        migrations.RemoveField(
            model_name="well",
            name="collections",
        ),
        migrations.AddField(
            model_name="biosample",
            name="_previous_runs",
            field=models.ManyToManyField(related_name="+", to="lamindb.run"),
        ),
        migrations.AddField(
            model_name="biosample",
            name="artifacts",
            field=models.ManyToManyField(
                related_name="biosamples", to="lamindb.artifact"
            ),
        ),
        migrations.AddField(
            model_name="biosample",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AddField(
            model_name="biosample",
            name="organism",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="biosamples",
                to="bionty.organism",
            ),
        ),
        migrations.AddField(
            model_name="biosample",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AddField(
            model_name="combinationtreatment",
            name="_previous_runs",
            field=models.ManyToManyField(related_name="+", to="lamindb.run"),
        ),
        migrations.AddField(
            model_name="combinationtreatment",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AddField(
            model_name="compoundtreatment",
            name="_previous_runs",
            field=models.ManyToManyField(related_name="+", to="lamindb.run"),
        ),
        migrations.AddField(
            model_name="compoundtreatment",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AddField(
            model_name="environmentaltreatment",
            name="_previous_runs",
            field=models.ManyToManyField(related_name="+", to="lamindb.run"),
        ),
        migrations.AddField(
            model_name="environmentaltreatment",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AddField(
            model_name="experiment",
            name="_previous_runs",
            field=models.ManyToManyField(related_name="+", to="lamindb.run"),
        ),
        migrations.AddField(
            model_name="experiment",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AddField(
            model_name="genetictreatment",
            name="_previous_runs",
            field=models.ManyToManyField(related_name="+", to="lamindb.run"),
        ),
        migrations.AddField(
            model_name="genetictreatment",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AddField(
            model_name="techsample",
            name="_previous_runs",
            field=models.ManyToManyField(related_name="+", to="lamindb.run"),
        ),
        migrations.AddField(
            model_name="techsample",
            name="artifacts",
            field=models.ManyToManyField(
                related_name="techsamples", to="lamindb.artifact"
            ),
        ),
        migrations.AddField(
            model_name="techsample",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AddField(
            model_name="treatmenttarget",
            name="_previous_runs",
            field=models.ManyToManyField(related_name="+", to="lamindb.run"),
        ),
        migrations.AddField(
            model_name="treatmenttarget",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AddField(
            model_name="well",
            name="_previous_runs",
            field=models.ManyToManyField(related_name="+", to="lamindb.run"),
        ),
        migrations.AddField(
            model_name="well",
            name="run",
            field=models.ForeignKey(
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="combinationtreatment",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="compoundtreatment",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="environmentaltreatment",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="experiment",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="genetictreatment",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="techsample",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="treatmenttarget",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="well",
            name="created_by",
            field=models.ForeignKey(
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.CreateModel(
            name="ArtifactBiosample",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("label_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links_biosample",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "biosample",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifact",
                        to="wetlab.biosample",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifactbiosample",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.LinkORM, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactCombinationTreatment",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("label_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links_combination_treatment",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "combinationtreatment",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifact",
                        to="wetlab.combinationtreatment",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifactcombinationtreatment",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.LinkORM, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactCompoundTreatment",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("label_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links_compound_treatment",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "compoundtreatment",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifact",
                        to="wetlab.compoundtreatment",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifactcompoundtreatment",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.LinkORM, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactEnvironmentalTreatment",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("label_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links_environmental_treatment",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.user",
                    ),
                ),
                (
                    "environmentaltreatment",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifact",
                        to="wetlab.environmentaltreatment",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifactenvironmentaltreatment",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.LinkORM, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactExperiment",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("label_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links_experiment",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.user",
                    ),
                ),
                (
                    "experiment",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifact",
                        to="wetlab.experiment",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifactexperiment",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.LinkORM, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactGeneticTreatment",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("label_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links_genetic_treatment",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifactgenetictreatment",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "genetictreatment",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifact",
                        to="wetlab.genetictreatment",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.run",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.LinkORM, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactTechsample",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("label_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links_techsample",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifacttechsample",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.run",
                    ),
                ),
                (
                    "techsample",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifact",
                        to="wetlab.techsample",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.LinkORM, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactTreatmentTarget",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("label_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links_treatment_target",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifacttreatmenttarget",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.run",
                    ),
                ),
                (
                    "treatmenttarget",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifact",
                        to="wetlab.treatmenttarget",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.LinkORM, models.Model),
        ),
        migrations.CreateModel(
            name="ArtifactWell",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("label_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links_well",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifactwell",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.run",
                    ),
                ),
                (
                    "well",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifact",
                        to="wetlab.well",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.LinkORM, models.Model),
        ),
        migrations.CreateModel(
            name="Compound",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=bionty.ids.ontology, max_length=8, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, max_length=256)),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                (
                    "abbr",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=32,
                        null=True,
                        unique=True,
                    ),
                ),
                ("synonyms", models.TextField(default=None, null=True)),
                ("description", models.TextField(default=None, null=True)),
                (
                    "_previous_runs",
                    models.ManyToManyField(related_name="+", to="lamindb.run"),
                ),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="compounds", to="lamindb.artifact"
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.user",
                    ),
                ),
                (
                    "parents",
                    models.ManyToManyField(
                        related_name="children", to="wetlab.compound"
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.run",
                    ),
                ),
                (
                    "source",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="bionty.source",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                models.Model,
                lamindb.models.HasParents,
                lamindb.models.CanCurate,
            ),
        ),
        migrations.CreateModel(
            name="ArtifactCompound",
            fields=[
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("label_ref_is_name", models.BooleanField(default=None, null=True)),
                ("feature_ref_is_name", models.BooleanField(default=None, null=True)),
                (
                    "artifact",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links_compound",
                        to="lamindb.artifact",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.user",
                    ),
                ),
                (
                    "feature",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifactcompound",
                        to="lamindb.feature",
                    ),
                ),
                (
                    "run",
                    models.ForeignKey(
                        default=lamindb.models.current_run,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="lamindb.run",
                    ),
                ),
                (
                    "compound",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="links_artifact",
                        to="wetlab.compound",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(lamindb.models.LinkORM, models.Model),
        ),
        migrations.AddField(
            model_name="compoundtreatment",
            name="compounds",
            field=models.ManyToManyField(
                related_name="compounds", to="wetlab.compound"
            ),
        ),
        migrations.DeleteModel(
            name="ExperimentType",
        ),
    ]



# Generated by Django 5.0.6 on 2024-05-16 09:02

import django.db.migrations.operations.special
import django.db.models.deletion
import django.utils.timezone
import lamindb.base.ids
import lamindb.base.users
from django.db import migrations, models

# Functions from the following migrations need manual copying.
# Move them and any dependencies into this file, then update the
# RunPython operations to refer to the local versions:
# wetlab.migrations.0013_


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("bionty", "0028_squashed"),
    ]

    operations = [
        migrations.CreateModel(
            name="Biosample",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=lamindb.base.ids.base62_12, max_length=12, unique=True
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        db_index=True, default=None, max_length=255, null=True
                    ),
                ),
                (
                    "batch",
                    models.CharField(
                        db_index=True, default=None, max_length=60, null=True
                    ),
                ),
                ("description", models.TextField(default=None, null=True)),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                (
                    "cell_lines",
                    models.ManyToManyField(
                        related_name="biosamples", to="bionty.cellline"
                    ),
                ),
                (
                    "cell_types",
                    models.ManyToManyField(
                        related_name="biosamples", to="bionty.celltype"
                    ),
                ),
                (
                    "collections",
                    models.ManyToManyField(
                        related_name="biosamples", to="lamindb.collection"
                    ),
                ),
                (
                    "diseases",
                    models.ManyToManyField(
                        related_name="biosamples", to="bionty.disease"
                    ),
                ),
                (
                    "tissues",
                    models.ManyToManyField(
                        related_name="biosamples", to="bionty.tissue"
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(models.Model, lamindb.models.CanCurate),
        ),
        migrations.CreateModel(
            name="ExperimentType",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=lamindb.base.ids.base62_4, max_length=4, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, default=None, max_length=255)),
                ("description", models.TextField(default=None, null=True)),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="created_experiment_types",
                        to="lamindb.user",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(models.Model, lamindb.models.CanCurate),
        ),
        migrations.CreateModel(
            name="Experiment",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=lamindb.base.ids.base62_8, max_length=8, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, default=None, max_length=255)),
                ("description", models.TextField(default=None, null=True)),
                ("date", models.DateField(db_index=True, default=None, null=True)),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="experiments", to="lamindb.artifact"
                    ),
                ),
                (
                    "collections",
                    models.ManyToManyField(
                        related_name="experiments", to="lamindb.collection"
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="created_experiments",
                        to="lamindb.user",
                    ),
                ),
                (
                    "experiment_type",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="experiments",
                        to="wetlab.experimenttype",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(models.Model, lamindb.models.CanCurate),
        ),
        migrations.CreateModel(
            name="Techsample",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=lamindb.base.ids.base62_12, max_length=12, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, default=None, max_length=255)),
                ("batch", models.CharField(db_index=True, default=None, max_length=60)),
                ("description", models.TextField(default=None, null=True)),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                (
                    "biosamples",
                    models.ManyToManyField(
                        related_name="techsamples", to="wetlab.biosample"
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="created_techsamples",
                        to="lamindb.user",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(models.Model, lamindb.models.CanCurate),
        ),
        migrations.CreateModel(
            name="TreatmentTarget",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=lamindb.base.ids.base62_8, max_length=8, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, default=None, max_length=60)),
                ("description", models.TextField(default=None, null=True)),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="treatment_targets", to="lamindb.artifact"
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="created_treatment_targets",
                        to="lamindb.user",
                    ),
                ),
                (
                    "genes",
                    models.ManyToManyField(
                        related_name="treatment_targets", to="bionty.gene"
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(models.Model, lamindb.models.CanCurate),
        ),
        migrations.CreateModel(
            name="Treatment",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=lamindb.base.ids.base62_12, max_length=12, unique=True
                    ),
                ),
                ("name", models.CharField(db_index=True, default=None, max_length=255)),
                (
                    "type",
                    models.CharField(
                        choices=[("genetic", "genetic"), ("chemical", "chemical")],
                        db_index=True,
                        max_length=20,
                    ),
                ),
                (
                    "system",
                    models.CharField(
                        choices=[
                            ("CRISPR Cas9", "CRISPR_Cas9"),
                            ("CRISPRi", "CRISPRi"),
                            ("CRISPRa", "CRISPRa"),
                            ("shRNA", "shRNA"),
                            ("siRNA", "siRNA"),
                            ("transgene", "transgene"),
                            ("transient transfection", "transient_transfection"),
                        ],
                        db_index=True,
                        default=None,
                        max_length=20,
                    ),
                ),
                ("description", models.TextField(default=None, null=True)),
                ("sequence", models.TextField(db_index=True, default=None, null=True)),
                (
                    "on_target_score",
                    models.FloatField(db_index=True, default=None, null=True),
                ),
                (
                    "off_target_score",
                    models.FloatField(db_index=True, default=None, null=True),
                ),
                (
                    "ontology_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                (
                    "pubchem_id",
                    models.CharField(
                        db_index=True, default=None, max_length=32, null=True
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                (
                    "artifacts",
                    models.ManyToManyField(
                        related_name="treatments", to="lamindb.artifact"
                    ),
                ),
                (
                    "collections",
                    models.ManyToManyField(
                        related_name="treatments", to="lamindb.collection"
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="created_treatments",
                        to="lamindb.user",
                    ),
                ),
                (
                    "targets",
                    models.ManyToManyField(
                        related_name="treatments", to="wetlab.treatmenttarget"
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(models.Model, lamindb.models.CanCurate),
        ),
        migrations.CreateModel(
            name="Well",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                (
                    "uid",
                    models.CharField(
                        default=lamindb.base.ids.base62_4, max_length=4, unique=True
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        db_index=True,
                        default=None,
                        max_length=32,
                        null=True,
                        unique=True,
                    ),
                ),
                ("row", models.CharField(default=None, max_length=4)),
                ("column", models.IntegerField()),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                (
                    "artifacts",
                    models.ManyToManyField(related_name="wells", to="lamindb.artifact"),
                ),
                (
                    "collections",
                    models.ManyToManyField(
                        related_name="wells", to="lamindb.collection"
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        default=lamindb.base.users.current_user_id,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="created_wells",
                        to="lamindb.user",
                    ),
                ),
            ],
            options={
                "unique_together": {("row", "column")},
            },
            bases=(models.Model, lamindb.models.CanCurate),
        ),
    ]



from typing import Literal

GeneticPerturbationSystem = Literal[
    "CRISPR-Cas9",
    "CRISPR-Cas13",
    "CRISPRi",
    "CRISPRa",
    "shRNA",
    "siRNA",
    "transgene",
    "transient-transfection",
]

BiologicType = Literal[
    "protein",
    "peptide",
    "antibody",
    "enzyme",
    "growth-factor",
    "cytokine",
    "hormone",
    "vaccine",
    "oligonucleotide",
    "co-culture",
]



# Generated by Django 5.2 on 2025-01-13 11:15

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wetlab", "0031_alter_biologic_space_alter_biosample_space_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="biologic",
            old_name="aux",
            new_name="_aux",
        ),
        migrations.RenameField(
            model_name="biosample",
            old_name="aux",
            new_name="_aux",
        ),
        migrations.RenameField(
            model_name="combinationperturbation",
            old_name="aux",
            new_name="_aux",
        ),
        migrations.RenameField(
            model_name="compound",
            old_name="aux",
            new_name="_aux",
        ),
        migrations.RenameField(
            model_name="compoundperturbation",
            old_name="aux",
            new_name="_aux",
        ),
        migrations.RenameField(
            model_name="donor",
            old_name="aux",
            new_name="_aux",
        ),
        migrations.RenameField(
            model_name="environmentalperturbation",
            old_name="aux",
            new_name="_aux",
        ),
        migrations.RenameField(
            model_name="experiment",
            old_name="aux",
            new_name="_aux",
        ),
        migrations.RenameField(
            model_name="geneticperturbation",
            old_name="aux",
            new_name="_aux",
        ),
        migrations.RenameField(
            model_name="perturbationtarget",
            old_name="aux",
            new_name="_aux",
        ),
        migrations.RenameField(
            model_name="techsample",
            old_name="aux",
            new_name="_aux",
        ),
        migrations.RenameField(
            model_name="well",
            old_name="aux",
            new_name="_aux",
        ),
    ]



# Generated by Django 5.2 on 2024-09-16 18:23

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("lamindb", "0066_alter_artifact__feature_values_and_more"),
        ("wetlab", "0022_remove_experiment_experiment_type_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="compoundtreatment",
            name="duration",
            field=models.DurationField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="environmentaltreatment",
            name="duration",
            field=models.DurationField(default=None, null=True),
        ),
        migrations.RunSQL(
            f"""
            INSERT INTO wetlab_artifactcombinationtreatment (artifact_id, combinationtreatment_id, feature_id, created_by_id, created_at)
            SELECT artifact_id, combinationtreatment_id, NULL, {1}, CURRENT_TIMESTAMP
            FROM wetlab_combinationtreatment_artifacts;
            """
        ),
        migrations.RunSQL(
            f"""
            INSERT INTO wetlab_artifactcompound (artifact_id, compound_id, feature_id, created_by_id, created_at)
            SELECT artifact_id, compound_id, NULL, {1}, CURRENT_TIMESTAMP
            FROM wetlab_compound_artifacts;
            """
        ),
        migrations.RunSQL(
            f"""
            INSERT INTO wetlab_artifactcompoundtreatment (artifact_id, compoundtreatment_id, feature_id, created_by_id, created_at)
            SELECT artifact_id, compoundtreatment_id, NULL, {1}, CURRENT_TIMESTAMP
            FROM wetlab_compoundtreatment_artifacts;
            """
        ),
        migrations.RunSQL(
            f"""
            INSERT INTO wetlab_artifactenvironmentaltreatment (artifact_id, environmentaltreatment_id, feature_id, created_by_id, created_at)
            SELECT artifact_id, environmentaltreatment_id, NULL, {1}, CURRENT_TIMESTAMP
            FROM wetlab_environmentaltreatment_artifacts;
            """
        ),
        migrations.RunSQL(
            f"""
            INSERT INTO wetlab_artifactexperiment (artifact_id, experiment_id, feature_id, created_by_id, created_at)
            SELECT artifact_id, experiment_id, NULL, {1}, CURRENT_TIMESTAMP
            FROM wetlab_experiment_artifacts;
            """
        ),
        migrations.RunSQL(
            f"""
            INSERT INTO wetlab_artifactgenetictreatment (artifact_id, genetictreatment_id, feature_id, created_by_id, created_at)
            SELECT artifact_id, genetictreatment_id, NULL, {1}, CURRENT_TIMESTAMP
            FROM wetlab_genetictreatment_artifacts;
            """
        ),
        migrations.RunSQL(
            f"""
            INSERT INTO wetlab_artifacttreatmenttarget (artifact_id, treatmenttarget_id, feature_id, created_by_id, created_at)
            SELECT artifact_id, treatmenttarget_id, NULL, {1}, CURRENT_TIMESTAMP
            FROM wetlab_treatmenttarget_artifacts;
            """
        ),
        migrations.RunSQL(
            f"""
            INSERT INTO wetlab_artifactwell (artifact_id, well_id, feature_id, created_by_id, created_at)
            SELECT artifact_id, well_id, NULL, {1}, CURRENT_TIMESTAMP
            FROM wetlab_well_artifacts;
            """
        ),
        migrations.RemoveField(
            model_name="combinationtreatment",
            name="artifacts",
        ),
        migrations.AddField(
            model_name="combinationtreatment",
            name="artifacts",
            field=models.ManyToManyField(
                related_name="combination_treatments",
                through="wetlab.ArtifactCombinationTreatment",
                to="lamindb.artifact",
            ),
        ),
        migrations.RemoveField(
            model_name="compound",
            name="artifacts",
        ),
        migrations.AddField(
            model_name="compound",
            name="artifacts",
            field=models.ManyToManyField(
                related_name="compounds",
                through="wetlab.ArtifactCompound",
                to="lamindb.artifact",
            ),
        ),
        migrations.RemoveField(
            model_name="compoundtreatment",
            name="artifacts",
        ),
        migrations.AddField(
            model_name="compoundtreatment",
            name="artifacts",
            field=models.ManyToManyField(
                related_name="compound_treatments",
                through="wetlab.ArtifactCompoundTreatment",
                to="lamindb.artifact",
            ),
        ),
        migrations.RemoveField(
            model_name="environmentaltreatment",
            name="artifacts",
        ),
        migrations.AddField(
            model_name="environmentaltreatment",
            name="artifacts",
            field=models.ManyToManyField(
                related_name="environmental_treatments",
                through="wetlab.ArtifactEnvironmentalTreatment",
                to="lamindb.artifact",
            ),
        ),
        migrations.RemoveField(
            model_name="experiment",
            name="artifacts",
        ),
        migrations.AddField(
            model_name="experiment",
            name="artifacts",
            field=models.ManyToManyField(
                related_name="experiments",
                through="wetlab.ArtifactExperiment",
                to="lamindb.artifact",
            ),
        ),
        migrations.RemoveField(
            model_name="genetictreatment",
            name="artifacts",
        ),
        migrations.AddField(
            model_name="genetictreatment",
            name="artifacts",
            field=models.ManyToManyField(
                related_name="genetic_treatments",
                through="wetlab.ArtifactGeneticTreatment",
                to="lamindb.artifact",
            ),
        ),
        migrations.RemoveField(
            model_name="treatmenttarget",
            name="artifacts",
        ),
        migrations.AddField(
            model_name="treatmenttarget",
            name="artifacts",
            field=models.ManyToManyField(
                related_name="treatment_targets",
                through="wetlab.ArtifactTreatmentTarget",
                to="lamindb.artifact",
            ),
        ),
        migrations.RemoveField(
            model_name="well",
            name="artifacts",
        ),
        migrations.AddField(
            model_name="well",
            name="artifacts",
            field=models.ManyToManyField(
                related_name="wells",
                through="wetlab.ArtifactWell",
                to="lamindb.artifact",
            ),
        ),
    ]



# Generated by Django 5.0.3 on 2024-09-20 11:08

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wetlab", "0023_compoundtreatment_duration_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="compound",
            name="chembl_id",
            field=models.CharField(
                db_index=True, default=None, max_length=32, null=True
            ),
        ),
    ]



# Generated by Django 5.0.6 on 2024-06-26 15:05

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wetlab", "0018_squashed"),
    ]

    operations = [
        migrations.AlterField(
            model_name="treatment",
            name="system",
            field=models.CharField(
                choices=[
                    ("CRISPR Cas9", "CRISPR_Cas9"),
                    ("CRISPRi", "CRISPRi"),
                    ("CRISPRa", "CRISPRa"),
                    ("shRNA", "shRNA"),
                    ("siRNA", "siRNA"),
                    ("transgene", "transgene"),
                    ("transient transfection", "transient_transfection"),
                ],
                db_index=True,
                default=None,
                max_length=32,
            ),
        ),
    ]



# Generated by Django 5.2 on 2024-11-15 09:38

import bionty.ids
import django.db.models.deletion
import lamindb.base.fields
import lamindb.base.ids
import lamindb.base.users
import lamindb.models
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0041_alter_artifactcellline_artifact_and_more"),
        (
            "lamindb",
            "0069_alter_artifact__accessor_alter_artifact__hash_type_and_more",
        ),
        ("wetlab", "0024_compound_chembl_id"),
    ]

    operations = [
        migrations.AlterField(
            model_name="artifactbiosample",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_biosample",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactbiosample",
            name="biosample",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="wetlab.biosample",
            ),
        ),
        migrations.AlterField(
            model_name="artifactbiosample",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactbiosample",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactbiosample",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactbiosample",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactbiosample",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactbiosample",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactbiosample",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcombinationtreatment",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_combination_treatment",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcombinationtreatment",
            name="combinationtreatment",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="wetlab.combinationtreatment",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcombinationtreatment",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactcombinationtreatment",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcombinationtreatment",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactcombinationtreatment",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcombinationtreatment",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactcombinationtreatment",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactcombinationtreatment",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcompound",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_compound",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcompound",
            name="compound",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="wetlab.compound",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcompound",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactcompound",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcompound",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactcompound",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcompound",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactcompound",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactcompound",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcompoundtreatment",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_compound_treatment",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcompoundtreatment",
            name="compoundtreatment",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="wetlab.compoundtreatment",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcompoundtreatment",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactcompoundtreatment",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcompoundtreatment",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactcompoundtreatment",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcompoundtreatment",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactcompoundtreatment",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactcompoundtreatment",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactenvironmentaltreatment",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_environmental_treatment",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactenvironmentaltreatment",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactenvironmentaltreatment",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactenvironmentaltreatment",
            name="environmentaltreatment",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="wetlab.environmentaltreatment",
            ),
        ),
        migrations.AlterField(
            model_name="artifactenvironmentaltreatment",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactenvironmentaltreatment",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactenvironmentaltreatment",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactenvironmentaltreatment",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactenvironmentaltreatment",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperiment",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_experiment",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperiment",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactexperiment",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperiment",
            name="experiment",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="wetlab.experiment",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperiment",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactexperiment",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperiment",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactexperiment",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactexperiment",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgenetictreatment",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_genetic_treatment",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgenetictreatment",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactgenetictreatment",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgenetictreatment",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactgenetictreatment",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgenetictreatment",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactgenetictreatment",
            name="genetictreatment",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="wetlab.genetictreatment",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgenetictreatment",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactgenetictreatment",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttechsample",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_techsample",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttechsample",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifacttechsample",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttechsample",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifacttechsample",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttechsample",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifacttechsample",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifacttechsample",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttechsample",
            name="techsample",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="wetlab.techsample",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttreatmenttarget",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_treatment_target",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttreatmenttarget",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifacttreatmenttarget",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttreatmenttarget",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifacttreatmenttarget",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttreatmenttarget",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifacttreatmenttarget",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifacttreatmenttarget",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttreatmenttarget",
            name="treatmenttarget",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="wetlab.treatmenttarget",
            ),
        ),
        migrations.AlterField(
            model_name="artifactwell",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_well",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactwell",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactwell",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactwell",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactwell",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactwell",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactwell",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactwell",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactwell",
            name="well",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="wetlab.well",
            ),
        ),
        migrations.AlterField(
            model_name="biosample",
            name="batch",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=60, null=True
            ),
        ),
        migrations.AlterField(
            model_name="biosample",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="biosample",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="biosample",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="biosample",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=255, null=True
            ),
        ),
        migrations.AlterField(
            model_name="biosample",
            name="organism",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="biosamples",
                to="bionty.organism",
            ),
        ),
        migrations.AlterField(
            model_name="biosample",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="biosample",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                default=lamindb.base.ids.base62_12,
                max_length=12,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="biosample",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="combinationtreatment",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="combinationtreatment",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="combinationtreatment",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="combinationtreatment",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=255
            ),
        ),
        migrations.AlterField(
            model_name="combinationtreatment",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="combinationtreatment",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="combinationtreatment",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                default=lamindb.base.ids.base62_12,
                max_length=12,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="combinationtreatment",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="compound",
            name="abbr",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=32,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="compound",
            name="chembl_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="compound",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="compound",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="compound",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="compound",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256
            ),
        ),
        migrations.AlterField(
            model_name="compound",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="compound",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="compound",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="compound",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="compound",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.ontology, max_length=8, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="compound",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="compoundtreatment",
            name="concentration",
            field=lamindb.base.fields.FloatField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="compoundtreatment",
            name="concentration_unit",
            field=lamindb.base.fields.CharField(
                blank=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="compoundtreatment",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="compoundtreatment",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="compoundtreatment",
            name="duration",
            field=lamindb.base.fields.DurationField(
                blank=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="compoundtreatment",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=255
            ),
        ),
        migrations.AlterField(
            model_name="compoundtreatment",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="compoundtreatment",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                default=lamindb.base.ids.base62_12,
                max_length=12,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="compoundtreatment",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="environmentaltreatment",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="environmentaltreatment",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="environmentaltreatment",
            name="duration",
            field=lamindb.base.fields.DurationField(
                blank=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="environmentaltreatment",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=255
            ),
        ),
        migrations.AlterField(
            model_name="environmentaltreatment",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="environmentaltreatment",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="environmentaltreatment",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                default=lamindb.base.ids.base62_12,
                max_length=12,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="environmentaltreatment",
            name="unit",
            field=lamindb.base.fields.CharField(
                blank=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="environmentaltreatment",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="environmentaltreatment",
            name="value",
            field=lamindb.base.fields.FloatField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="experiment",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="experiment",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="experiment",
            name="date",
            field=lamindb.base.fields.DateField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="experiment",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="experiment",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=255
            ),
        ),
        migrations.AlterField(
            model_name="experiment",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="experiment",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                default=lamindb.base.ids.base62_8,
                max_length=8,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="experiment",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="genetictreatment",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="genetictreatment",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="genetictreatment",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=255
            ),
        ),
        migrations.AlterField(
            model_name="genetictreatment",
            name="off_target_score",
            field=lamindb.base.fields.FloatField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="genetictreatment",
            name="on_target_score",
            field=lamindb.base.fields.FloatField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="genetictreatment",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="genetictreatment",
            name="sequence",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="genetictreatment",
            name="system",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32
            ),
        ),
        migrations.AlterField(
            model_name="genetictreatment",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                default=lamindb.base.ids.base62_12,
                max_length=12,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="genetictreatment",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="techsample",
            name="batch",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=60
            ),
        ),
        migrations.AlterField(
            model_name="techsample",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="techsample",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="techsample",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="techsample",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=255
            ),
        ),
        migrations.AlterField(
            model_name="techsample",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="techsample",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                default=lamindb.base.ids.base62_12,
                max_length=12,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="techsample",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="treatmenttarget",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="treatmenttarget",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="treatmenttarget",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="treatmenttarget",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=60
            ),
        ),
        migrations.AlterField(
            model_name="treatmenttarget",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="treatmenttarget",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                default=lamindb.base.ids.base62_8,
                max_length=8,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="treatmenttarget",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="well",
            name="column",
            field=lamindb.base.fields.IntegerField(blank=True),
        ),
        migrations.AlterField(
            model_name="well",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="well",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="well",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=32,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="well",
            name="row",
            field=lamindb.base.fields.CharField(blank=True, default=None, max_length=4),
        ),
        migrations.AlterField(
            model_name="well",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="well",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                default=lamindb.base.ids.base62_4,
                max_length=4,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="well",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
    ]



# Generated by Django 5.2 on 2024-08-01 13:22

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wetlab", "0020_treatmenttarget_pathways_treatmenttarget_proteins_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="compoundtreatment",
            name="duration_unit",
            field=models.CharField(default=None, max_length=32, null=True),
        ),
        migrations.AlterField(
            model_name="environmentaltreatment",
            name="duration_unit",
            field=models.CharField(default=None, max_length=32, null=True),
        ),
        migrations.AlterField(
            model_name="genetictreatment",
            name="system",
            field=models.CharField(db_index=True, default=None, max_length=32),
        ),
    ]



from __future__ import annotations

import re
from typing import TYPE_CHECKING, Literal, Optional, Tuple

import bionty as bt
import lamindb as ln
import pandas as pd
from lamin_utils import colors, logger

try:
    from cellxgene_lamin import CellxGeneFields
    from cellxgene_lamin import Curate as CellxGeneCurate
except ImportError:
    CellxGeneFields = "CellxGeneFields"

    class CellxGeneCurate:  # type: ignore
        def __init__(self, *args, **kwargs):
            raise RuntimeError(
                "cellxgene_lamin is not installed. Please install it to use PertCurator."
            )


from .models import (
    Biologic,
    Compound,
    Donor,
    EnvironmentalPerturbation,
    GeneticPerturbation,
    PerturbationTarget,
)

if TYPE_CHECKING:
    import anndata as ad
    from lamindb.base.types import FieldAttr


class ValidationError(SystemExit):
    """Validation error."""

    pass


class PertValidatorUnavailable(SystemExit):
    """Curator for perturbation data when dependencies are unavailable."""

    pass


class ValueUnit:
    """Base class for handling value-unit combinations."""

    @staticmethod
    def parse_value_unit(value: str, is_dose: bool = True) -> tuple[str, str] | None:
        """Parse a string containing a value and unit into a tuple."""
        if not isinstance(value, str) or not value.strip():
            return None

        value = str(value).strip()
        match = re.match(r"^(\d*\.?\d{0,1})\s*([a-zA-ZμµΜ]+)$", value)

        if not match:
            raise ValueError(
                f"Invalid format: {value}. Expected format: number with max 1 decimal place + unit"
            )

        number, unit = match.groups()
        formatted_number = f"{float(number):.1f}"

        if is_dose:
            standardized_unit = DoseHandler.standardize_unit(unit)
            if not DoseHandler.validate_unit(standardized_unit):
                raise ValueError(
                    f"Invalid dose unit: {unit}. Must be convertible to one of: nM, μM, mM, M"
                )
        else:
            standardized_unit = TimeHandler.standardize_unit(unit)
            if not TimeHandler.validate_unit(standardized_unit):
                raise ValueError(
                    f"Invalid time unit: {unit}. Must be convertible to one of: h, m, s, d, y"
                )

        return formatted_number, standardized_unit


class DoseHandler:
    """Handler for dose-related operations."""

    VALID_UNITS = {"nM", "μM", "µM", "mM", "M"}
    UNIT_MAP = {
        "nm": "nM",
        "NM": "nM",
        "um": "μM",
        "UM": "μM",
        "μm": "μM",
        "μM": "μM",
        "µm": "μM",
        "µM": "μM",
        "mm": "mM",
        "MM": "mM",
        "m": "M",
        "M": "M",
    }

    @classmethod
    def validate_unit(cls, unit: str) -> bool:
        """Validate if the dose unit is acceptable."""
        return unit in cls.VALID_UNITS

    @classmethod
    def standardize_unit(cls, unit: str) -> str:
        """Standardize dose unit to standard formats."""
        return cls.UNIT_MAP.get(unit, unit)

    @classmethod
    def validate_values(cls, values: pd.Series) -> list:
        """Validate pert_dose values with strict case checking."""
        errors = []

        for idx, value in values.items():
            if pd.isna(value):
                continue

            if isinstance(value, (int, float)):
                errors.append(
                    f"Row {idx} - Missing unit for dose: {value}. Must include a unit (nM, μM, mM, M)"
                )
                continue

            try:
                ValueUnit.parse_value_unit(value, is_dose=True)
            except ValueError as e:
                errors.append(f"Row {idx} - {str(e)}")

        return errors


class TimeHandler:
    """Handler for time-related operations."""

    VALID_UNITS = {"h", "m", "s", "d", "y"}

    @classmethod
    def validate_unit(cls, unit: str) -> bool:
        """Validate if the time unit is acceptable."""
        return unit == unit.lower() and unit in cls.VALID_UNITS

    @classmethod
    def standardize_unit(cls, unit: str) -> str:
        """Standardize time unit to standard formats."""
        if unit.startswith("hr"):
            return "h"
        elif unit.startswith("min"):
            return "m"
        elif unit.startswith("sec"):
            return "s"
        return unit[0].lower()

    @classmethod
    def validate_values(cls, values: pd.Series) -> list:
        """Validate pert_time values."""
        errors = []

        for idx, value in values.items():
            if pd.isna(value):
                continue

            if isinstance(value, (int, float)):
                errors.append(
                    f"Row {idx} - Missing unit for time: {value}. Must include a unit (h, m, s, d, y)"
                )
                continue

            try:
                ValueUnit.parse_value_unit(value, is_dose=False)
            except ValueError as e:
                errors.append(f"Row {idx} - {str(e)}")

        return errors


class PertCurator(CellxGeneCurate):
    """Curator flow for Perturbation data."""

    PERT_COLUMNS = {"compound", "genetic", "biologic", "physical"}

    def __init__(
        self,
        adata: ad.AnnData,
        var_index: FieldAttr = bt.Gene.ensembl_gene_id,
        organism: Literal["human", "mouse"] = "human",
        pert_dose: bool = True,
        pert_time: bool = True,
        *,
        verbosity: str = "hint",
        cxg_schema_version: Literal["5.0.0", "5.1.0"] = "5.1.0",
        using_key: str | None = None,
    ):
        """Initialize the curator with configuration and validation settings."""
        if isinstance(CellxGeneFields, str):
            CellxGeneCurate()

        self._pert_time = pert_time
        self._pert_dose = pert_dose

        self._validate_initial_data(adata)
        self._setup_configuration(adata)

        self._setup_sources(adata, using_key)
        self._setup_compound_source()

        super().__init__(
            adata=adata,
            var_index=var_index,
            categoricals=self.PT_CATEGORICALS,
            using_key=using_key,
            defaults=self.PT_DEFAULT_VALUES,
            verbosity=verbosity,
            organism=organism,
            extra_sources=self.PT_SOURCES,
            schema_version=cxg_schema_version,
        )

    def _setup_configuration(self, adata: ad.AnnData):
        """Set up default configuration values."""
        self.PT_DEFAULT_VALUES = CellxGeneFields.OBS_FIELD_DEFAULTS | {
            "cell_line": "unknown",
            "pert_target": "unknown",
        }

        self.PT_CATEGORICALS = CellxGeneFields.OBS_FIELDS | {
            k: v
            for k, v in {
                "cell_line": bt.CellLine.name,
                "pert_target": PerturbationTarget.name,
                "pert_genetic": GeneticPerturbation.name,
                "pert_compound": Compound.name,
                "pert_biologic": Biologic.name,
                "pert_physical": EnvironmentalPerturbation.name,
            }.items()
            if k in adata.obs.columns
        }
        # if "donor_id" in self.PT_CATEGORICALS:
        #     self.PT_CATEGORICALS["donor_id"] = Donor.name

    def _setup_sources(self, adata: ad.AnnData, using_key: str):
        """Set up data sources."""
        self.PT_SOURCES = {}
        # if "cell_line" in adata.obs.columns:
        #     self.PT_SOURCES["cell_line"] = (
        #         bt.Source.using(using_key).filter(name="depmap").first()
        #     )
        if "pert_compound" in adata.obs.columns:
            self.PT_SOURCES["pert_compound"] = (
                bt.Source.using(using_key)
                .filter(entity="wetlab.Compound", name="chebi")
                .first()
            )

    def _validate_initial_data(self, adata: ad.AnnData):
        """Validate the initial data structure."""
        self._validate_required_columns(adata)
        self._validate_perturbation_types(adata)

    def _validate_required_columns(self, adata: ad.AnnData):
        """Validate required columns are present."""
        if "pert_target" not in adata.obs.columns:
            if (
                "pert_name" not in adata.obs.columns
                or "pert_type" not in adata.obs.columns
            ):
                raise ValidationError(
                    "either 'pert_target' or both 'pert_name' and 'pert_type' must be present"
                )
        else:
            if "pert_name" not in adata.obs.columns:
                logger.warning(
                    "no 'pert' column found in adata.obs, will only curate 'pert_target'"
                )
            elif "pert_type" not in adata.obs.columns:
                raise ValidationError("both 'pert' and 'pert_type' must be present")

    def _validate_perturbation_types(self, adata: ad.AnnData):
        """Validate perturbation types."""
        if "pert_type" in adata.obs.columns:
            data_pert_types = set(adata.obs["pert_type"].unique())
            invalid_pert_types = data_pert_types - self.PERT_COLUMNS
            if invalid_pert_types:
                raise ValidationError(
                    f"invalid pert_type found: {invalid_pert_types}!\n"
                    f"    → allowed values: {self.PERT_COLUMNS}"
                )
            self._process_perturbation_types(adata, data_pert_types)

    def _process_perturbation_types(self, adata: ad.AnnData, pert_types: set):
        """Process and map perturbation types."""
        for pert_type in pert_types:
            col_name = "pert_" + pert_type
            adata.obs[col_name] = adata.obs["pert_name"].where(
                adata.obs["pert_type"] == pert_type, None
            )
            if adata.obs[col_name].dtype.name == "category":
                adata.obs[col_name].cat.remove_unused_categories()
            logger.important(f"mapped 'pert_name' to '{col_name}'")

    def _setup_compound_source(self):
        """Set up the compound source with muted logging."""
        with logger.mute():
            chebi_source = bt.Source.filter(
                entity="wetlab.Compound", name="chebi"
            ).first()
            if not chebi_source:
                Compound.add_source(
                    bt.Source.filter(entity="Drug", name="chebi").first()
                )

    def validate(self) -> bool:
        """Validate the AnnData object."""
        validated = super().validate()

        if self._pert_dose:
            validated &= self._validate_dose_column()
        if self._pert_time:
            validated &= self._validate_time_column()

        self._validated = validated

        # sort columns
        first_columns = [
            "pert_target",
            "pert_genetic",
            "pert_compound",
            "pert_biologic",
            "pert_physical",
            "pert_dose",
            "pert_time",
            "organism",
            "cell_line",
            "cell_type",
            "disease",
            "tissue_type",
            "tissue",
            "assay",
            "suspension_type",
            "donor_id",
            "sex",
            "self_reported_ethnicity",
            "development_stage",
            "pert_name",
            "pert_type",
        ]
        sorted_columns = [
            col for col in first_columns if col in self._adata.obs.columns
        ] + [col for col in self._adata.obs.columns if col not in first_columns]
        # must assign to self._df to ensure .standardize works correctly
        self._df = self._adata.obs[sorted_columns]
        self._adata.obs = self._df
        return validated

    def standardize(self, key: str) -> pd.DataFrame:
        """Standardize the AnnData object."""
        super().standardize(key)
        self._adata.obs = self._df

    def _validate_dose_column(self) -> bool:
        """Validate the dose column."""
        if not ln.Feature.filter(name="pert_dose").exists():
            ln.Feature(name="pert_dose", dtype="str").save()

        dose_errors = DoseHandler.validate_values(self._adata.obs["pert_dose"])
        if dose_errors:
            self._log_validation_errors("pert_dose", dose_errors)
            return False
        return True

    def _validate_time_column(self) -> bool:
        """Validate the time column."""
        if not ln.Feature.filter(name="pert_time").exists():
            ln.Feature(name="pert_time", dtype="str").save()

        time_errors = TimeHandler.validate_values(self._adata.obs["pert_time"])
        if time_errors:
            self._log_validation_errors("pert_time", time_errors)
            return False
        return True

    def _log_validation_errors(self, column: str, errors: list):
        """Log validation errors with formatting."""
        errors_print = "\n    ".join(errors)
        logger.warning(
            f"invalid {column} values found!\n    {errors_print}\n"
            f"    → run {colors.cyan('standardize_dose_time()')}"
        )

    def standardize_dose_time(self) -> pd.DataFrame:
        """Standardize dose and time values."""
        standardized_df = self._adata.obs.copy()

        if "pert_dose" in self._adata.obs.columns:
            standardized_df = self._standardize_column(
                standardized_df, "pert_dose", is_dose=True
            )

        if "pert_time" in self._adata.obs.columns:
            standardized_df = self._standardize_column(
                standardized_df, "pert_time", is_dose=False
            )

        self._adata.obs = standardized_df
        return standardized_df

    def _standardize_column(
        self, df: pd.DataFrame, column: str, is_dose: bool
    ) -> pd.DataFrame:
        """Standardize values in a specific column."""
        for idx, value in self._adata.obs[column].items():
            if pd.isna(value) or (
                isinstance(value, str) and (not value.strip() or value.lower() == "nan")
            ):
                df.at[idx, column] = None
                continue

            try:
                num, unit = ValueUnit.parse_value_unit(value, is_dose=is_dose)
                df.at[idx, column] = f"{num}{unit}"
            except ValueError:
                continue

        return df
